package de.cweiske.jOnlineControl.connectors;

import de.cweiske.jOnlineControl.Status;

import java.net.*;
import java.io.*;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
 * Fritz!Box driver
 *
 * @author Christian Weiske <cweiske@cweiske.de>
 */
public class FritzBox extends AddressConnector implements Connector
{
    protected static final int port = 49000;
    protected static final String defaultIp = "fritz.box";

    protected static int nTimeout        = 500;
    protected static int nConnectTimeout = 500;



    public boolean connect()
    {
        return true;
    }



    public boolean disconnect()
    {
        return true;
    }



    public Status getStatus()
    {
        String ip = extractValueFromTag(
                sendUPnPRequest("urn:schemas-upnp-org:service:WANIPConnection:1", "GetExternalIPAddress"),
                "NewExternalIPAddress"
        );

        String addons = sendUPnPRequest(
                "urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1",
                "GetAddonInfos"
        );
        String dns1 = extractValueFromTag(addons, "NewDNSServer1");
        String dns2 = extractValueFromTag(addons, "NewDNSServer2");

        if (ip == null) {
            return null;
        }

        Status status = new Status();

        status.setConnected(!ip.equals("0.0.0.0"));

        try {
            status.setWanIp(InetAddress.getByName(ip));
            if (dns1 != null) {
                status.setPrimaryDns(InetAddress.getByName(dns1));
            }
            if (dns2 != null) {
                status.setSecondaryDns(InetAddress.getByName(dns2));
            }
        } catch (UnknownHostException e) {
            return null;
        }

        return status;
    }



    /**
     * Sends a UPnP request to the box and returns the answer (xml)
     *
     * @param schema XML schema to use
     * @param request Request tag to use
     * @return xml string
     */
    protected String sendUPnPRequest(String schema, String request)
    {
        if (host == null) {
            return null;
        }

        String httpRequest =
                "POST /upnp/control/WANCommonIFC1 HTTP/1.1\n"
                + "Content-Type: text/xml; charset=\"utf-8\"\n"
                + "HOST: " + host.getHostAddress() + ":" + port + "\n"
                + "Content-Length: 0\n"
                + "SOAPACTION: \"" + schema + "#" + request + "\"\n"
                + "\n"
                + "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
                + "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">\n"
                + "<s:Body><u:" + request + " xmlns:u=" + schema + " /></s:Body>\n"
                + "</s:Envelope>\n";

        Socket sock;
        InputStream in;
        OutputStream out;
        try {
            sock = new Socket();
            sock.setSoTimeout(nTimeout);
            sock.connect(new InetSocketAddress(host, port), nTimeout);

            in = sock.getInputStream();
            out = sock.getOutputStream();
        } catch (NoRouteToHostException e) {
//            System.out.println( "Can't connect to router. Check IP.");
            return null;
        } catch (SocketException e) {
            //interface isn't set up
            return null;
        } catch (SocketTimeoutException e) {
            //network cable not attached
            return null;
        } catch (IOException e ) {
            e.printStackTrace();
            return null;
        }

        try {
            out.write(httpRequest.getBytes());
        } catch (IOException e ) {
            e.printStackTrace();
            return null;
        }
        BufferedReader stdIn = new BufferedReader(new InputStreamReader(in));

        StringBuffer answer = new StringBuffer(500);
        //read the response
        try {
            String strLine;
            while ((strLine = stdIn.readLine()) != null) {
                answer.append(strLine).append("\n");
            }
            sock.close();

        } catch (IOException e) {
            return null;
        }

        return answer.toString();
    }//protected String sendUPnPRequest(String schema, String request)



    protected String extractValueFromTag(String content, String tagname)
    {
        if (content == null) {
            return null;
        }

        Pattern ptTag   = Pattern.compile("<" + tagname + ">(.*)</" + tagname + ">");

        Matcher m;
        if ((m = ptTag.matcher(content)) != null && m.find()) {
            return m.group(1);
        } else {
            return null;
        }
    }//protected String extractValueFromTag(String content, String tagname)



    public String getDefaultIp()
    {
        return defaultIp;
    }
    
}//public class FritzBox extends AddressConnector implements Connector